#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# dataport-schuposh-acls
# listener module
#
# Copyright 2022 Univention GmbH
#
# https://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <https://www.gnu.org/licenses/>.

from __future__ import absolute_import

import listener
import subprocess
import univention.debug as ud
import univention.admin.uldap
from typing import List, Any

ldap_base = listener.configRegistry["ldap/base"]
server_role = listener.configRegistry["server/role"]
group_dn = "cn=iqsh-admins,cn=groups,{}".format(ldap_base)

name = "dataport-schuposh-acls"
description = "Add schools to admin group import permissions"
filter = "(&(ucsschoolRole=school:school:*)(objectClass=ucsschoolOrganizationalUnit))"
attributes = []


def handler(dn, new, old):
    # type: (str, dict, dict) -> None

    if not server_role == "domaincontroller_master":
        # ud.debug(ud.LISTENER, ud.WARN, "{}: Not on a DC master, not doing anything".format(name))
        return
    ud.debug(ud.LISTENER, ud.WARN, "{}: DN {}".format(name, dn))
    listener.setuid(0)
    lo, po = univention.admin.uldap.getAdminConnection()

    current_ou = new["ou"][0]

    modlist = []  # type: List[List[Any]]
    # add objectClass "ucsschoolImportGroup" if missing
    existing_grp_objectclasses = lo.getAttr(group_dn, "objectClass")
    if b"ucsschoolImportGroup" not in existing_grp_objectclasses:
        modlist.append(("objectClass", b"", b"ucsschoolImportGroup"))
    # add ou to "ucsschoolImportSchool" if not present
    existing_school_permissions = lo.getAttr(group_dn, "ucsschoolImportSchool")
    if current_ou not in existing_school_permissions:
        new_school_permissions = [school for school in existing_school_permissions]
        new_school_permissions.append(current_ou)
        modlist.append(
            (
                "ucsschoolImportSchool",
                existing_school_permissions,
                new_school_permissions,
            )
        )
    # perform changes if there are any
    if modlist:
        ud.debug(
            ud.LISTENER,
            ud.WARN,
            "{}: LO modify with group {} and modlist {}".format(
                name, group_dn, modlist
            ),
        )
        lo.modify(group_dn, modlist)

    # convert byte string to string
    current_ou_str = current_ou.decode("UTF-8")

    school_import_admin_group_name = "import_admins-{}".format(current_ou_str.lower())
    school_import_admin_group_dn = "cn={},cn=groups,ou={},{}".format(
        school_import_admin_group_name, current_ou_str, ldap_base
    )
    existing_school_import_admin_group = lo.get(school_import_admin_group_dn)
    if not existing_school_import_admin_group:
        ud.debug(
            ud.LISTENER,
            ud.WARN,
            "{}: Import Admins group does not exist. Creating with udm: {}".format(
                name, school_import_admin_group_dn
            ),
        )
        subprocess.call(
            [
                "udm",
                "groups/group",
                "create",
                "--position",
                "cn=groups,ou={},{}".format(current_ou_str, ldap_base),
                "--set",
                "name={}".format(school_import_admin_group_name),
            ]
        )

    existing_school_import_admin_group = lo.get(school_import_admin_group_dn)
    if existing_school_import_admin_group:
        existing_objectClasses = lo.getAttr(school_import_admin_group_dn, "objectClass")
        existing_school_permissions = lo.getAttr(
            school_import_admin_group_dn, "ucsschoolImportSchool"
        )
        if current_ou not in existing_school_permissions:
            new_school_permissions = [school for school in existing_school_permissions]
            new_school_permissions.append(current_ou)
            new_objectClasses = [objectClass for objectClass in existing_objectClasses]
            new_objectClasses.append(b"ucsschoolImportGroup")
            modlist = [
                (
                    "ucsschoolImportSchool",
                    existing_school_permissions,
                    new_school_permissions,
                ),
                ("objectClass", existing_objectClasses, new_objectClasses),
            ]
            ud.debug(
                ud.LISTENER,
                ud.WARN,
                "{}: LO modify with group {} and modlist {}".format(
                    name, school_import_admin_group_dn, modlist
                ),
            )
            lo.modify(school_import_admin_group_dn, modlist)

        existing_policy_reference = lo.getAttr(
            school_import_admin_group_dn, "univentionPolicyReference"
        )
        if not existing_policy_reference:
            existing_objectClasses = lo.getAttr(
                school_import_admin_group_dn, "objectClass"
            )
            new_objectClasses = [objectClass for objectClass in existing_objectClasses]
            new_objectClasses.append(b"univentionPolicyReference")
            # modlist expects byte strings now
            modlist = [
                (
                    "univentionPolicyReference",
                    b"",
                    ("cn=dataport-schuposh-import-admins,cn=UMC,cn=policies,{}".format(
                        ldap_base
                    )).encode(),
                ),
                ("objectClass", existing_objectClasses, new_objectClasses),
            ]
            ud.debug(
                ud.LISTENER,
                ud.WARN,
                "{}: LO modify with group {} and modlist {}".format(
                    name, school_import_admin_group_dn, modlist
                ),
            )
            lo.modify(school_import_admin_group_dn, modlist)

        existing_school_roles = lo.getAttr(
            school_import_admin_group_dn, "ucsschoolImportRole"
        )
        new_school_roles = [role for role in existing_school_roles]
        for role in [b"student", b"teacher"]:
            if role not in existing_school_roles:
                new_school_roles.append(role)
        modlist = [("ucsschoolImportRole", existing_school_roles, new_school_roles)]
        ud.debug(
            ud.LISTENER,
            ud.WARN,
            "{}: LO modify with group {} and modlist {}".format(
                name, school_import_admin_group_dn, modlist
            ),
        )
        lo.modify(school_import_admin_group_dn, modlist)
    else:
        ud.debug(
            ud.LISTENER,
            ud.ERROR,
            "{}: Group does not exist after udm create! DN: {}".format(
                name, school_import_admin_group_dn
            ),
        )

    ud.debug(ud.LISTENER, ud.WARN, "{}: Done for DN {}".format(name, dn))


def postrun():
    if not server_role == "domaincontroller_master":
        return
    # Run the ucr commit and slapd restart only if no new change arrives within 15 seconds
    listener.setuid(0)
    try:
        ud.debug(
            ud.LISTENER,
            ud.WARN,
            "{}: Calling ucr commit and slapd restart script".format(name),
        )
        subprocess.call(["/etc/univention/templates/scripts/dataport-schuposh-acls"])
    finally:
        listener.unsetuid()
